"""Command-line interface."""
import click

from ecs_tasks_ops import ecs_data
from ecs_tasks_ops import ecs_facade
from ecs_tasks_ops import pretty_json
from ecs_tasks_ops import pretty_table


@click.group()
@click.option("-x", "--debug/--no-debug", default=False)
@click.option("-j", "--output-json", "output_json", is_flag=True, default=False)
@click.version_option()
@click.pass_context
def main(ctx, debug, output_json) -> None:
    """Ecs Tasks Ops."""
    ctx.ensure_object(dict)
    ctx.obj["DEBUG"] = debug
    # TODO: Format output with json
    ctx.obj["OUT_JSON"] = output_json


@main.command("clusters")
@click.pass_context
def main_clusters(ctx):
    """Clusters information."""
    clusters = ecs_data.get_clusters()
    if ctx.obj["OUT_JSON"]:
        click.echo(pretty_json.dumps(clusters))
    else:
        click.echo(pretty_table.tabulate_list_json(clusters, fields_to=7))


@main.command("services")
@click.option("-c", "--cluster-name", default="default", help="Cluster name")
@click.pass_context
def main_services(ctx, cluster_name):
    """Services defined in a cluster."""
    try:
        services_info = ecs_data.get_services(cluster_name)
        if ctx.obj["OUT_JSON"]:
            click.echo(pretty_json.dumps(services_info))
        else:
            click.echo(
                pretty_table.tabulate_list_json_keys(
                    services_info,
                    [
                        "serviceArn",
                        "serviceName",
                        "status",
                        "runningCount",
                        "desiredCount",
                    ],
                )
            )

    except ecs_facade.ecs_client().exceptions.ClusterNotFoundException:
        click.secho(f"Cluster {cluster_name} not found", fg="red")
        return []
    except Exception:
        click.secho("There has been an error.", fg="red")
        return []


@main.command("container-instances")
@click.option("-c", "--cluster-name", default="default", help="Cluster name")
@click.pass_context
def main_container_instances(ctx, cluster_name):
    """Container instances defined in a cluster."""
    try:
        container_instances_info = ecs_data.get_containers_instances(cluster_name)
        if ctx.obj["OUT_JSON"]:
            click.echo(pretty_json.dumps(container_instances_info))
        else:
            click.echo(
                pretty_table.tabulate_list_json_keys(
                    container_instances_info, ["ec2InstanceId", "versionInfo"]
                )
            )

    except ecs_facade.ecs_client().exceptions.ClusterNotFoundException:
        click.secho(f"Cluster {cluster_name} not found", fg="red")
        return []
    except Exception:
        click.secho("There has been an error.", fg="red")
        return []


@main.command("tasks")
@click.option("-c", "--cluster-name", default="default", help="Cluster name")
@click.option("-s", "--service-name", help="Service name")
@click.option("-i", "--container-instance", help="Container instance")
@click.pass_context
def main_tasks(ctx, cluster_name, service_name, container_instance):
    """Set tasks defined in a cluster."""
    try:
        if not service_name and not container_instance:
            tasks_info = ecs_data.get_tasks_cluster(cluster_name)
        elif service_name:
            tasks_info = ecs_data.get_tasks_service(cluster_name, service_name)
        elif container_instance:
            tasks_info = ecs_data.get_tasks_container_instance(
                cluster_name, container_instance
            )

        if ctx.obj["OUT_JSON"]:
            click.echo(pretty_json.dumps(tasks_info))
        else:
            click.echo(
                pretty_table.tabulate_list_json_keys(
                    tasks_info,
                    [
                        "taskArn",
                        "ec2InstanceId",
                        "availabilityZone",
                        "memory",
                        "cpu",
                        "desiredStatus",
                        "healthStatus",
                        "lastStatus",
                    ],
                )
            )

    except ecs_facade.ecs_client().exceptions.ClusterNotFoundException:
        click.secho(f"Cluster {cluster_name} not found", fg="red")
        return []
    except Exception:
        click.secho("There has been an error.", fg="red")
        return []


@main.command("containers")
@click.option("-c", "--cluster-name", default="default", help="Cluster name")
@click.option("-s", "--service-name", help="Service name", required=True)
@click.option("-d", "--docker-name", help="Docker container name")
@click.pass_context
def main_containers(ctx, cluster_name, service_name, docker_name):
    """Get docker containers defined in a cluster."""
    try:
        containers_info = ecs_data.get_containers_service(cluster_name, service_name)

        if docker_name:
            containers_info = [c for c in containers_info if c["name"] == docker_name]

        if ctx.obj["OUT_JSON"]:
            click.echo(pretty_json.dumps(containers_info))
        else:
            click.echo(
                pretty_table.tabulate_list_json_keys(
                    containers_info,
                    [
                        "image",
                        "ec2InstanceId",
                        "name",
                        "memory",
                        "cpu",
                        "runtimeId",
                        "healthStatus",
                        "lastStatus",
                    ],
                )
            )

    except ecs_facade.ecs_client().exceptions.ClusterNotFoundException:
        click.secho(f"Cluster {cluster_name} not found", fg="red")
        return []
    except ecs_facade.ecs_client().exceptions.ServiceNotFoundException:
        click.secho(f"Service {service_name} not found", fg="red")
        return []
    except Exception:
        click.secho("There has been an error.", fg="red")
        return []


if __name__ == "__main__":
    # main(prog_name="ecs-tasks-ops")  # pragma: no cover
    pass



"""Module to create beautiful tables from dicts."""
from tabulate import tabulate


def tabulate_list_json(
    data=(), fields_from=None, fields_to=None, empty_msg="No data to show"
) -> str:
    """Generate a table with information from a list of json."""
    if not data:
        return empty_msg

    keys = list(data[0].keys())
    if fields_from and fields_to:
        keys = keys[fields_from:fields_to]
    elif fields_from:
        keys = keys[fields_from:]
    elif fields_to:
        keys = keys[:fields_to]
    return tabulate_list_json_keys(data=data, keys=keys, empty_msg=empty_msg)


def tabulate_list_json_keys(data=(), keys=(), empty_msg="No data to show"):
    """Generate a table from a list of json where you can specify keys to show."""
    if not data:
        return empty_msg

    headers = {k: k for k in keys}
    return tabulate(
        [{k: v for k, v in c.items() if k in headers} for c in data], headers=headers
    )



"""Qt5 Tree Model for ecs."""
from PyQt5 import QtCore
from PyQt5 import QtWidgets

from ecs_tasks_ops import ecs_conf
from ecs_tasks_ops import ecs_data
from ecs_tasks_ops import ecs_facade
from ecs_tasks_ops import ecs_ssh
from ecs_tasks_ops import pretty_json


class ECSTreeItem(QtWidgets.QTreeWidgetItem):
    """Tree Widgets Item for ECS elements."""

    def __init__(self, name, identifier, detail_type, detail, parent=None):
        """Initialization of basic ECS element, with name, arn_id, type and json data."""
        super(ECSTreeItem, self).__init__(parent, [name, identifier])
        self.name = name
        self.identifier = identifier
        self.detail_type = detail_type
        self.detail = detail

    def refresh_children(self):
        """Refresh list of children elements."""
        pass

    def get_attributes(self):
        """Get a list of basic attributes for detail view."""
        return [
            ("Name", self.name),
            ("Identifier", self.identifier),
            ("Detail Type", self.detail_type),
        ]

    def clear_children(self):
        """Clear list of children elements."""
        for i in reversed(range(self.childCount())):
            self.removeChild(self.child(i))

    def get_context_menu(self, menu):
        """Get basic context menu with mouse right-button."""
        menu.addSeparator()
        menu.addAction("Refresh Children", self.refresh_children)
        menu.addAction("Show Detail", self.command_show_detail)

    def command_show_detail(self):
        """Update detail view with this elements."""
        self.treeWidget().command_show_detail(self)


class ECSClusterTreeItem(ECSTreeItem):
    """Tree Widgets Item for Clusters elements."""

    def __init__(self, detail, parent=None):
        """Initialize Cluster item."""
        super(ECSClusterTreeItem, self).__init__(
            name=detail["clusterName"],
            identifier=detail["clusterArn"],
            detail_type="cluster",
            detail=detail,
            parent=parent,
        )

    def refresh_children(self):
        """Refresh cluster children with available options."""
        self.clear_children()
        self.addChild(ECSListTasksClusterTreeItem(self.detail, self))
        self.addChild(ECSListServicesClusterTreeItem(self.detail, self))
        self.addChild(ECSListContainersClusterTreeItem(self.detail, self))

    def get_attributes(self):
        """Get a list of basic attributes for cluters."""
        return [
            ("Name", self.name),
            ("Identifier", self.identifier),
            ("Detail Type", self.detail_type),
            ("Status", self.detail["status"]),
            ("Active Services", self.detail["activeServicesCount"]),
            ("Running Tasks", self.detail["runningTasksCount"]),
            ("Pending Tasks", self.detail["pendingTasksCount"]),
            ("Containers", self.detail["registeredContainerInstancesCount"]),
        ]


class ECSListServicesClusterTreeItem(ECSTreeItem):
    """Tree Widgets Item for List of Services of a cluster."""

    def __init__(self, detail, parent=None):
        """Initilize info to get a list of services for a cluster."""
        super(ECSListServicesClusterTreeItem, self).__init__(
            name=f"Services on '{detail['clusterName']}'",
            identifier=detail["clusterName"],
            detail_type="list_services",
            detail=detail,
            parent=parent,
        )

    def refresh_children(self):
        """Get a list of services for a cluster."""
        self.clear_children()
        for service in ecs_data.get_services(self.identifier):
            self.addChild(ECSServiceTreeItem(service, self))


class ECSListTasksClusterTreeItem(ECSTreeItem):
    """Tree Widgets Item for List of Tasks of a cluster."""

    def __init__(self, detail, parent=None):
        """Initialize info to get a list of tasks for a cluster."""
        super(ECSListTasksClusterTreeItem, self).__init__(
            name=f"Tasks on '{detail['clusterName']}'",
            identifier=detail["clusterName"],
            detail_type="list_services",
            detail=detail,
            parent=parent,
        )

    def refresh_children(self):
        """Get a list of tasks for a cluster."""
        self.clear_children()
        for task in ecs_data.get_tasks_cluster(self.identifier):
            self.addChild(ECSTaskTreeItem(task, self))


class ECSListContainersClusterTreeItem(ECSTreeItem):
    """Tree Widgets Item for List of Containers Instances of a cluster."""

    def __init__(self, detail, parent=None):
        """Initialize info to get a list of container instances for a cluster."""
        super(ECSListContainersClusterTreeItem, self).__init__(
            name=f"Containers on '{detail['clusterName']}'",
            identifier=detail["clusterName"],
            detail_type="list_services",
            detail=detail,
            parent=parent,
        )

    def refresh_children(self):
        """Get a list of container instances for a cluster."""
        self.clear_children()
        for container in ecs_data.get_containers_instances(self.identifier):
            self.addChild(ECSContainerTreeItem(container, self.identifier, self))


class ECSServiceTreeItem(ECSTreeItem):
    """Tree Widgets Item for Services elements."""

    def __init__(self, detail, parent=None):
        """Contains information about a service."""
        super(ECSServiceTreeItem, self).__init__(
            name=detail["serviceName"],
            identifier=detail["serviceArn"],
            detail_type="service",
            detail=detail,
            parent=parent,
        )
        self.cluster_identifier = detail["clusterArn"]

    def refresh_children(self):
        """Get a list of tasks for this service."""
        self.clear_children()
        for task in ecs_data.get_tasks_service(
            self.cluster_identifier, self.identifier
        ):
            self.addChild(ECSTaskTreeItem(task, self))

    def get_attributes(self):
        """Detail attributes view for this service."""
        deployment_config = self.detail["deploymentConfiguration"]
        min_bracket = deployment_config["minimumHealthyPercent"]
        max_bracket = deployment_config["maximumPercent"]

        return [
            ("Name", self.name),
            ("Identifier", self.identifier),
            ("Detail Type", self.detail_type),
            ("Status", self.detail["status"]),
            ("Task Definition", self.detail["taskDefinition"]),
            ("Running", self.detail["runningCount"]),
            ("Pending", self.detail["pendingCount"]),
            ("Desired", self.detail["desiredCount"]),
            (
                "Redeployment bracket",
                "Min: " + str(min_bracket) + "%, Max: " + str(max_bracket) + "%",
            ),
        ]

    def get_context_menu(self, menu):
        """Context menu for a service item."""
        menu.addAction("Show Events", self.command_service_show_events)
        menu.addAction("Force Restart Service", self.command_service_restart)
        super(ECSServiceTreeItem, self).get_context_menu(menu)

    def command_service_restart(self):
        """Send a restart command for this service."""
        self.treeWidget().command_service_restart(self)

    def command_service_show_events(self):
        """Open a list of events produced in this services."""
        self.treeWidget().command_service_show_events(self)


class ECSTaskTreeItem(ECSTreeItem):
    """Tree Widgets Item for Tasks elements."""

    def __init__(self, detail, parent=None):
        """Contains information about a task."""
        super(ECSTaskTreeItem, self).__init__(
            name=detail["name"],
            identifier=detail["taskArn"],
            detail_type="task",
            detail=detail,
            parent=parent,
        )
        self.cluster_identifier = self.detail["clusterArn"]

    def refresh_children(self):
        """Get a list of docker containers of this task."""
        self.clear_children()
        for task in ecs_data.get_containers_tasks(
            self.cluster_identifier, self.identifier
        ):
            self.addChild(
                ECSDockerContainerTreeItem(
                    task, self.identifier, self.cluster_identifier, self
                )
            )

    def get_attributes(self):
        """Get a list of attributes for detail view."""
        return [
            ("Name", self.name),
            ("Identifier", self.identifier),
            ("Detail Type", self.detail_type),
            ("Cluster Arn", self.detail["clusterArn"]),
            ("Status", self.detail["lastStatus"]),
            ("Desired Status", self.detail["desiredStatus"]),
            ("EC2 Instance", self.detail["ec2InstanceId"]),
            ("Availability Zone", self.detail["availabilityZone"]),
            ("Connectivity", self.detail["connectivity"]),
            ("Task Definition", self.detail["taskDefinitionArn"]),
            (
                "Container Instance ID",
                self.detail["containerInstanceArn"].split("/", 1)[1],
            ),
            ("N. Docker images", len(self.detail["containers"])),
            ("Networks", "\n".join(self.detail["networks"])),
            ("Connectivity Time", self.detail["connectivityAt"]),
        ]

    def get_context_menu(self, menu):
        """Context menu for this task item."""
        menu.addAction("Stop Task", self.command_task_stop)
        menu.addAction("SSH Instance Container", self.command_container_ssh)
        menu.addAction("Docker Log (First Task)", self.command_task_log)
        super(ECSTaskTreeItem, self).get_context_menu(menu)

    def command_task_stop(self):
        """Send stop command to ECS for this task."""
        self.treeWidget().command_task_stop(self)

    def command_container_ssh(self):
        """Access to the continer instance of this tasks through ssh."""
        self.treeWidget().command_container_ssh(self)

    def command_task_log(self):
        """Show log info for the first docker container of this tasks."""
        self.treeWidget().command_task_log(self)


class ECSContainerTreeItem(ECSTreeItem):
    """Tree Widgets Item for Container Instances elements."""

    def __init__(self, detail, cluster_identifier, parent=None):
        """Contains information about container instances."""
        super(ECSContainerTreeItem, self).__init__(
            name=detail["ec2InstanceId"],
            identifier=detail["containerInstanceArn"],
            detail_type="container",
            detail=detail,
            parent=parent,
        )
        self.cluster_identifier = cluster_identifier

    def refresh_children(self):
        """Get a list of tasks of this container instance."""
        self.clear_children()
        for task in ecs_data.get_tasks_container_instance(
            self.cluster_identifier, self.identifier
        ):
            self.addChild(ECSTaskTreeItem(task, self))

    def get_attributes(self):
        """Get a list of attributes for detail view."""
        return [
            ("Name", self.name),
            ("Identifier", self.identifier),
            ("Detail Type", self.detail_type),
            ("Status", self.detail["status"]),
            ("EC2 Instance Id", self.detail["ec2InstanceId"]),
            ("Running Tasks", self.detail["runningTasksCount"]),
            ("Pending Tasks", self.detail["pendingTasksCount"]),
            ("AMI Id", self.detail["ami_id"]),
            ("Instance Type", self.detail["instance_type"]),
            ("Availability Zone", self.detail["availability_zone"]),
            (
                "Memory",
                "Available: "
                + str(self.detail["available_memory"])
                + " Total: "
                + str(self.detail["total_memory"]),
            ),
            (
                "CPU",
                "Available: "
                + str(self.detail["available_cpu"])
                + " Total: "
                + str(self.detail["total_cpu"]),
            ),
            ("Taken ports", self.detail["taken_ports"]),
        ]

    def get_context_menu(self, menu):
        """Context menu for container instances."""
        menu.addAction("SSH Instance Container", self.command_container_ssh)
        super(ECSContainerTreeItem, self).get_context_menu(menu)

    def command_container_ssh(self):
        """Access to this container instance through ssh."""
        self.treeWidget().command_container_ssh(self)


class ECSDockerContainerTreeItem(ECSTreeItem):
    """Tree Widgets Item for Docker Containers elements."""

    def __init__(self, detail, task_identifier, cluster_identifier, parent=None):
        """Contains information about docker container."""
        super(ECSDockerContainerTreeItem, self).__init__(
            name=detail["name"],
            identifier=detail["containerArn"],
            detail_type="docker_container",
            detail=detail,
            parent=parent,
        )
        self.cluster_identifier = cluster_identifier
        self.task_identifier = task_identifier

    # def refresh_children(self):
    #     self.clear_children()
    #     for task in ecs_data.get_tasks_container_instance(self.cluster_identifier, self.identifier):
    #         self.addChild(ECSTaskTreeItem(task, self))

    def get_attributes(self):
        """Get a list of attributes for detail view."""
        return [
            ("Name", self.name),
            ("Identifier", self.identifier),
            ("Detail Type", self.detail_type),
            ("Container Arn", self.detail["containerArn"]),
            ("Status", self.detail["lastStatus"]),
            ("Health Status", self.detail["healthStatus"]),
            ("Docker id", self.detail["runtimeId"]),
            ("Docker Image", self.detail["image"]),
            ("CPU", self.detail["cpu"]),
            # ('Memory Reservation', self.detail['memoryReservation']),
            ("Instance ID", self.detail["ec2InstanceId"]),
            ("Networks", self.detail["networks"]),
        ]

    def get_context_menu(self, menu):
        """Context menu for docker containers."""
        menu.addAction("SSH Instance Container", self.command_container_ssh)
        menu.addAction("Docker log", self.command_docker_log)
        menu.addAction("Docker exec", self.command_docker_exec)
        super(ECSDockerContainerTreeItem, self).get_context_menu(menu)

    def command_container_ssh(self):
        """Access to container instances server which contains this docker container."""
        self.treeWidget().command_container_ssh(self)

    def command_docker_log(self):
        """Show log information of this docker container."""
        self.treeWidget().command_docker_log(self)

    def command_docker_exec(self):
        """Exec a command on this docker container."""
        self.treeWidget().command_docker_exec(self)


class ECSElementsTreeWidget(QtWidgets.QTreeWidget):
    """Tree Widgets for all ECS elements."""

    sig_status_changed = QtCore.pyqtSignal(str)
    sig_command_show_detail = QtCore.pyqtSignal(QtWidgets.QTreeWidgetItem)
    sig_command_service_restart = QtCore.pyqtSignal(QtWidgets.QTreeWidgetItem)
    sig_command_service_show_events = QtCore.pyqtSignal(QtWidgets.QTreeWidgetItem)
    sig_command_task_stop = QtCore.pyqtSignal(QtWidgets.QTreeWidgetItem)
    sig_command_task_log = QtCore.pyqtSignal(QtWidgets.QTreeWidgetItem)
    sig_command_container_ssh = QtCore.pyqtSignal(QtWidgets.QTreeWidgetItem)
    sig_command_docker_log = QtCore.pyqtSignal(QtWidgets.QTreeWidgetItem)
    sig_command_docker_exec = QtCore.pyqtSignal(QtWidgets.QTreeWidgetItem)

    def __init__(self, parent=None):
        """Initialize tree widgets."""
        super(ECSElementsTreeWidget, self).__init__(parent)
        self.reload_cluster_info()
        self.currentItemChanged.connect(
            lambda item: self.show_status_on_selection(item)
        )
        self.itemActivated.connect(lambda item: item.refresh_children())
        self.itemDoubleClicked.connect(lambda item: item.refresh_children())

    def contextMenuEvent(self, event):
        """Open context menu depends on item options."""
        item = self.itemAt(event.pos())
        if item:
            menu = QtWidgets.QMenu(self)
            item.get_context_menu(menu)
            menu.exec_(event.globalPos())
        else:
            super(ECSElementsTreeWidget, self).contextMenuEvent(event)

    def keyPressEvent(self, e):
        """Process key pressed on this widget."""
        # if e.key() == QtCore.Qt.Key_Escape:
        #     self.close()

        super(ECSElementsTreeWidget, self).keyPressEvent(e)

    @QtCore.pyqtSlot()
    def reload_cluster_info(self):
        """Reload a list of clusters."""
        self.clear()
        for cluster in ecs_data.get_clusters():
            self.addTopLevelItem(ECSClusterTreeItem(cluster))

    def show_status_on_selection(self, item):
        """Update status bar with item info."""
        if item:
            self.sig_status_changed.emit(f"Selecting {item.name}: {item.identifier}")

    def command_show_detail(self, item):
        """Show attributes on detail view widget."""
        if item and isinstance(item, ECSTreeItem):
            self.sig_status_changed.emit(
                f"Show details for {item.name}: {item.identifier}"
            )
            self.sig_command_show_detail.emit(item)

    def command_service_restart(self, item):
        """Restart a service command."""
        if item and isinstance(item, ECSTreeItem):
            self.sig_status_changed.emit(
                f"Restarting service {item.name}: {item.identifier}"
            )
            self.sig_command_service_restart.emit(item)

    def command_task_stop(self, item):
        """Stop a task command."""
        if item and isinstance(item, ECSTreeItem):
            self.sig_status_changed.emit(
                f"Stopping task {item.name}: {item.identifier}"
            )
            self.sig_command_task_stop.emit(item)

    def command_task_log(self, item):
        """Show log of a task."""
        if item and isinstance(item, ECSTreeItem):
            self.sig_status_changed.emit(f"Log for task {item.name}: {item.identifier}")
            self.sig_command_task_log.emit(item)

    def command_container_ssh(self, item):
        """Access to container instance."""
        if item and isinstance(item, ECSTreeItem):
            self.sig_status_changed.emit(
                f"Access to ssh for {item.name}: {item.identifier}"
            )
            self.sig_command_container_ssh.emit(item)

    def command_docker_log(self, item):
        """Show log of docker container."""
        if item and isinstance(item, ECSTreeItem):
            self.sig_status_changed.emit(
                f"Docker log for {item.name}: {item.identifier}"
            )
            self.sig_command_docker_log.emit(item)

    def command_docker_exec(self, item):
        """Execute command on docker container."""
        if item and isinstance(item, ECSTreeItem):
            self.sig_status_changed.emit(
                f"Docker exec for {item.name}: {item.identifier}"
            )
            self.sig_command_docker_exec.emit(item)

    def command_service_show_events(self, item):
        """Show events of a service."""
        if item and isinstance(item, ECSTreeItem):
            self.sig_status_changed.emit(
                f"Show Events for {item.name}: {item.identifier}"
            )
            self.sig_command_service_show_events.emit(item)

    # @QtCore.pyqtSignal(str)
    # def operation_status(self, message):
    #     self.emit


class ECSAttributesTreeWidget(QtWidgets.QTreeWidget):
    """Widget to show item attributes."""

    sig_status_changed = QtCore.pyqtSignal(str)

    def __init__(self, parent=None):
        """Initialize attributes view."""
        super(ECSAttributesTreeWidget, self).__init__(parent)
        self.currentItemChanged.connect(
            lambda item: self.show_status_on_selection(item)
        )

    def show_status_on_selection(self, item):
        """Update status bar on selection."""
        if item:
            self.sig_status_changed.emit(f"{item.text(0)}: {item.text(1)}")

    @QtCore.pyqtSlot(QtWidgets.QTreeWidgetItem)
    def update_attributes(self, item):
        """Show a list of attributes of a selected ecs item."""
        self.clear()
        if item and isinstance(item, ECSTreeItem):
            for attr in item.get_attributes():
                self.addTopLevelItem(QtWidgets.QTreeWidgetItem([str(a) for a in attr]))


class ECSTabView(QtWidgets.QTabWidget):
    """Tab widgets for commands results."""

    def __init__(self, parent=None):
        """Initialize tab widget."""
        super(ECSTabView, self).__init__(parent)
        self.tabCloseRequested.connect(self.info_tab_closed)

    @QtCore.pyqtSlot(QtWidgets.QTreeWidgetItem)
    def show_detail(self, item):
        """Show json info of a ecs element."""
        if item and item.detail and isinstance(item, ECSTreeItem):
            tab_id = self.addTab(ShowResult(pretty_json.dumps(item.detail)), item.name)
            self.setCurrentIndex(tab_id)

    @QtCore.pyqtSlot(QtWidgets.QTreeWidgetItem)
    def container_ssh(self, item):
        """Open ssh terminal to access a container instance."""
        if item and item.detail and isinstance(item, ECSTreeItem):
            bash_command = ecs_ssh.ssh_cmd_container_instance(item.detail)
            tab_id = self.addTab(EmbTerminal(bash_command), item.name)
            self.setCurrentIndex(tab_id)

    @QtCore.pyqtSlot(QtWidgets.QTreeWidgetItem)
    def service_events(self, item):
        """Show a list of events for a service."""
        if item and item.detail and isinstance(item, ECSServiceTreeItem):
            tab_id = self.addTab(
                ShowServiceEvents(item.detail["events"]), f"Events from {item.name}"
            )
            self.setCurrentIndex(tab_id)

    @QtCore.pyqtSlot(QtWidgets.QTreeWidgetItem)
    def task_stop(self, item):
        """Send task stop command."""
        if item and item.detail and isinstance(item, ECSTaskTreeItem):
            question = QtWidgets.QMessageBox.question(
                self, f"Stopping {item.name}", f"Are you sure to stop {item.name}?"
            )
            if question == QtWidgets.QMessageBox.Yes:
                self.task_stop_ok(item)

    def task_stop_ok(self, item):
        """Show result of stop operation."""
        task_stopped = ecs_facade.stop_task(
            item.cluster_identifier, item.identifier, "Stopped from ECS Taks Operations"
        )
        tab_id = self.addTab(
            ShowResult(pretty_json.dumps(task_stopped)),
            f"Stopping {item.name}",
        )
        self.setCurrentIndex(tab_id)

    @QtCore.pyqtSlot(QtWidgets.QTreeWidgetItem)
    def service_restart(self, item):
        """Restart a service."""
        if item and item.detail and isinstance(item, ECSServiceTreeItem):
            question = QtWidgets.QMessageBox.question(
                self,
                f"Restart {item.name}",
                f"Are you sure to force restart {item.name}?",
            )
            if question == QtWidgets.QMessageBox.Yes:
                self.service_restart_ok(item)

    def service_restart_ok(self, item):
        """Show result of restart a service."""
        service_restarted = ecs_facade.restart_service(
            item.cluster_identifier, item.identifier, True
        )
        tab_id = self.addTab(
            ShowResult(pretty_json.dumps(service_restarted)),
            f"Force Restart {item.name}",
        )
        self.setCurrentIndex(tab_id)

    @QtCore.pyqtSlot(QtWidgets.QTreeWidgetItem)
    def task_log(self, item):
        """Open log terminal for a task (first docker container)."""
        if item and item.detail and isinstance(item, ECSTaskTreeItem):
            bash_command = ecs_ssh.ssh_cmd_task_log(item.detail)
            tab_id = self.addTab(EmbTerminal(bash_command), item.name)
            self.setCurrentIndex(tab_id)

    @QtCore.pyqtSlot(QtWidgets.QTreeWidgetItem)
    def docker_container_log(self, item):
        """Open log terminal for a docker container."""
        if item and item.detail and isinstance(item, ECSDockerContainerTreeItem):
            bash_command = ecs_ssh.ssh_cmd_docker_container_log(item.detail)
            tab_id = self.addTab(EmbTerminal(bash_command), item.name)
            self.setCurrentIndex(tab_id)

    @QtCore.pyqtSlot(QtWidgets.QTreeWidgetItem)
    def docker_container_exec(self, item):
        """Open terminal executing a command on docker container."""
        if item and item.detail and isinstance(item, ECSDockerContainerTreeItem):
            commands = ecs_conf.cfg.get("commands", ["/bin/sh"])
            command_on_docker, ok = QtWidgets.QInputDialog.getItem(
                self,
                "Command to execute on docker",
                "Command:",
                commands,
                editable=True,
            )
            if ok:
                bash_command = ecs_ssh.ssh_cmd_docker_container_exec(
                    item.detail, command_on_docker, True
                )
                tab_id = self.addTab(
                    EmbTerminal(bash_command), f"{command_on_docker} on {item.name}"
                )
                self.setCurrentIndex(tab_id)

    def info_tab_closed(self, index):
        """Close a tab."""
        self.widget(index).close()
        self.removeTab(index)


class ShowResult(QtWidgets.QWidget):
    """Show result widgets with text information."""

    def __init__(self, code_str, parent=None):
        """Initialize show result widget."""
        super(ShowResult, self).__init__(parent)

        layout = QtWidgets.QVBoxLayout()
        text = QtWidgets.QTextBrowser()
        text.setPlainText(code_str)
        layout.addWidget(text)

        self.setLayout(layout)


class EmbTerminal(QtWidgets.QWidget):
    """Open a embedded terminal widget."""

    def __init__(self, bash_command, parent=None):
        """Initilize parameters and open a terminal."""
        super(EmbTerminal, self).__init__(parent)
        self.process = QtCore.QProcess(self)
        self.terminal = QtWidgets.QWidget(self)
        self.command = QtWidgets.QLineEdit(self)
        self.command.setReadOnly(True)
        self.command.setText(bash_command)
        layout = QtWidgets.QVBoxLayout(self)
        layout.addWidget(self.command)
        layout.addWidget(self.terminal)

        # env = QtCore.QProcessEnvironment.systemEnvironment()
        # env.insert("TERM", "xterm")
        # self.process.setProcessEnvironment(env)

        # Log errors
        # self.process.error.connect(self.log_error)

        # Works also with urxvt:
        terminal_args = []
        if bash_command:
            terminal_args = ["-e", "bash", "-c", bash_command] + terminal_args
        self.process.start(
            "urxvt", ["-embed", str(int(self.terminal.winId()))] + terminal_args
        )

    def closeEvent(self, event):
        """Close terminal and process."""
        self.process.terminate()

    def log_error(self, error):
        """Show log error information."""
        print(error)


class ShowServiceEvents(QtWidgets.QWidget):
    """Widget for a list of events."""

    def __init__(self, events, parent=None):
        """Initialize this widget for a list of events."""
        super(ShowServiceEvents, self).__init__(parent)

        layout = QtWidgets.QVBoxLayout(self)
        table = QtWidgets.QTableWidget(self)
        table.setRowCount(len(events))
        table.setColumnCount(2)
        table.setHorizontalHeaderLabels(["Date", "Message"])
        table.horizontalHeader().setSectionResizeMode(
            QtWidgets.QHeaderView.ResizeToContents
        )
        # table.horizontalHeader().setCascadingSectionResizes(True) # Only valid for QHeaderView.Interactive
        for row, e in enumerate(events):
            table.setItem(
                row,
                0,
                QtWidgets.QTableWidgetItem(
                    e["createdAt"].strftime("%Y-%m-%d %H:%M:%S")
                ),
            )
            table.setItem(row, 1, QtWidgets.QTableWidgetItem(e["message"]))

        layout.addWidget(table)
        self.setLayout(layout)



"""Generate ssh commands to access to ECS resources."""


def ssh_cmd_container_instance(detail) -> str:
    """SSH command to access a ecs2 instance by id."""
    return f"TERM=xterm ssh {detail['ec2InstanceId']}"


def ssh_cmd_task_log(detail) -> str:
    """SSH command to access the first docker instance logs of a task."""
    return f"TERM=xterm ssh {detail['ec2InstanceId']} docker logs -f --tail=100 {detail['containers'][0]['runtimeId']}"


def ssh_cmd_task_exec(detail, command_on_docker, wait_press_key=None) -> str:
    """SSH command to execute a command inside the first docker containter of a task."""
    wait_cmd = ""
    if wait_press_key:
        wait_cmd = "; echo 'Press a key'; read q"
    return (
        f"TERM=xterm ssh -t {detail['ec2InstanceId']} docker exec -ti {detail['containers'][0]['runtimeId']} {command_on_docker}"
        + wait_cmd
    )


def ssh_cmd_docker_container_log(detail) -> str:
    """SSH command to access a docker instance logs."""
    return f"TERM=xterm ssh {detail['ec2InstanceId']} docker logs -f --tail=100 {detail['runtimeId']}"


def ssh_cmd_docker_container_exec(
    detail, command_on_docker, wait_press_key=None
) -> str:
    """SSH command to execute a command inside a docker containter."""
    wait_cmd = ""
    if wait_press_key:
        wait_cmd = "; echo 'Press a key'; read q"
    return (
        f"TERM=xterm ssh -t {detail['ec2InstanceId']} docker exec -ti {detail['runtimeId']} {command_on_docker}"
        + wait_cmd
    )



"""Qt5 UI Initialization."""
import click

from ecs_tasks_ops import ecs_conf
from ecs_tasks_ops_qt5 import qt5_gui


@click.version_option()
def main() -> None:
    """Open Qt5 User Interface."""
    ecs_conf.load_config()
    qt5_gui.main_gui()


if __name__ == "__main__":
    pass



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['ecs_tasks_ops', 'ecs_tasks_ops_qt5']

package_data = \
{'': ['*']}

install_requires = \
['PyQt5>=5.15.1,<6.0.0',
 'boto3>=1.15.13,<2.0.0',
 'click>=7,<9',
 'moto[ecs,ec2]>=1.3.16,<3.0.0',
 'tabulate>=0.8.7,<0.9.0']

entry_points = \
{'console_scripts': ['ecs-tasks-ops = ecs_tasks_ops.__main__:main',
                     'ecs-tasks-ops-qt5 = ecs_tasks_ops_qt5.__main__:main']}

setup_kwargs = {
    'name': 'ecs-tasks-ops',
    'version': '0.3.1',
    'description': 'Ecs Tasks Ops',
    'long_description': 'ECS Tasks Ops\n=============\n\n|PyPI| |Python Version| |License|\n\n|Read the Docs| |Tests| |Codecov|\n\n|pre-commit| |Black|\n\n.. |PyPI| image:: https://img.shields.io/pypi/v/ecs-tasks-ops.svg\n   :target: https://pypi.org/project/ecs-tasks-ops/\n   :alt: PyPI\n.. |Python Version| image:: https://img.shields.io/pypi/pyversions/ecs-tasks-ops\n   :target: https://pypi.org/project/ecs-tasks-ops\n   :alt: Python Version\n.. |License| image:: https://img.shields.io/pypi/l/ecs-tasks-ops\n   :target: https://opensource.org/licenses/MIT\n   :alt: License\n.. |Read the Docs| image:: https://img.shields.io/readthedocs/ecs-tasks-ops/latest.svg?label=Read%20the%20Docs\n   :target: https://ecs-tasks-ops.readthedocs.io/\n   :alt: Read the documentation at https://ecs-tasks-ops.readthedocs.io/\n.. |Tests| image:: https://github.com/ppalazon/ecs-tasks-ops/workflows/Tests/badge.svg\n   :target: https://github.com/ppalazon/ecs-tasks-ops/actions?workflow=Tests\n   :alt: Tests\n.. |Codecov| image:: https://codecov.io/gh/ppalazon/ecs-tasks-ops/branch/main/graph/badge.svg?token=zaz1KPR73Q\n   :target: https://codecov.io/gh/ppalazon/ecs-tasks-ops\n   :alt: Codecov\n.. |pre-commit| image:: https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white\n   :target: https://github.com/pre-commit/pre-commit\n   :alt: pre-commit\n.. |Black| image:: https://img.shields.io/badge/code%20style-black-000000.svg\n   :target: https://github.com/psf/black\n   :alt: Black\n\n\nFeatures\n--------\n\n* Application GUI to manage ECS resources\n* Use your home aws credentials from ~/.aws/credentials\n* Get information and attributes for each task, service or instance container\n* Connect through SSH to container instances or docker container\n* Show logs for each docker container\n* Show ECS events for a service\n* Force restart for a service\n\nRequirements\n------------\n\n* Python 3.10\n* `boto3 <https://pypi.org/project/boto3/>`_\n* `click <https://pypi.org/project/click/>`_\n* `tabulate <https://pypi.org/project/tabulate/>`_\n* `PyQt5 <https://pypi.org/project/PyQt5/>`_\n* `moto <https://pypi.org/project/moto/>`_\n* uxrvt\n\n\nInstallation\n------------\n\nYou can install *Ecs Tasks Ops* via pip_ from PyPI_:\n\n.. code:: console\n\n   $ pip install ecs-tasks-ops\n\n\nConfiguration\n-------------\n\nAWS Access\n^^^^^^^^^^\n\nThis application uses your aws credentials to connect to your ECS, so you need to configure your credentials.\n\nSet up credentials (in e.g. ``~/.aws/credentials``)\n\n.. code:: ini\n\n   [default]\n   aws_access_key_id = YOUR_KEY\n   aws_secret_access_key = YOUR_SECRET\n\nThen, you set up a default region (in e.g. ``~/.aws/config``)\n\n.. code:: ini\n\n   [default]\n   region=us-east-1\n\nYou can read more about it in `boto3 <https://pypi.org/project/boto3/>`_\n\n``ssh`` over ``ssm``\n^^^^^^^^^^^^^^^^^^^^\n\nIf you want to access to containers instances or docker container through ``ssh``, you must configurate ``ssm`` in your EC2 machines.\nThat\'s because ``ecs-tasks-ops`` use its instance id as machine identifier on ``ssh`` command. For example, ``ssh i-0123456789ABCDE``.\nI use `ssh-over-ssm <https://github.com/elpy1/ssh-over-ssm>`_ tool to configure ``ssh`` over ``ssm`` to connect to instances.\n\nGUI Usage\n---------\n\nYou can open the ``qt5`` application, using the following command\n\n.. code:: console\n\n   ecs-tasks-ops-qt5\n\nCLI Usage\n---------\n\nYou can open the command line with ``ecs-tasks-ops`` command. This is the help menu:\n\n.. code::\n\n   Usage: ecs-tasks-ops [OPTIONS] COMMAND [ARGS]...\n\n      Ecs Tasks Ops.\n\n   Options:\n      -x, --debug / --no-debug\n      -j, --output-json\n      --version                 Show the version and exit.\n      --help                    Show this message and exit.\n\n   Commands:\n      clusters             Clusters information.\n      container-instances  Container instances defined in a cluster.\n      containers           Get docker containers defined in a cluster.\n      services             Services defined in a cluster.\n      tasks                Set tasks defined in a cluster.\n\nBy default, the output format is in a table, but you can get original ``json`` format with ``-j`` option.\nYou can filter json output with `jq <https://stedolan.github.io/jq/>`_ tool:\n\n.. code:: console\n\n   $ ecs-tasks-ops -j clusters | jq "."\n\nContributing\n------------\n\nContributions are very welcome.\nTo learn more, see the `Contributor Guide`_.\n\n\nLicense\n-------\n\nDistributed under the terms of the MIT_ license,\n*Ecs Tasks Ops* is free and open source software.\n\n\nIssues\n------\n\nIf you encounter any problems,\nplease `file an issue`_ along with a detailed description.\n\n\nCredits\n-------\n\nThis project was generated from `@cjolowicz`_\'s `Hypermodern Python Cookiecutter`_ template.\n\n\n.. _@cjolowicz: https://github.com/cjolowicz\n.. _Cookiecutter: https://github.com/audreyr/cookiecutter\n.. _MIT: http://opensource.org/licenses/MIT\n.. _PyPI: https://pypi.org/\n.. _Hypermodern Python Cookiecutter: https://github.com/cjolowicz/cookiecutter-hypermodern-python\n.. _file an issue: https://github.com/ppalazon/ecs-tasks-ops/issues\n.. _pip: https://pip.pypa.io/\n.. github-only\n.. _Contributor Guide: CONTRIBUTING.rst\n',
    'author': 'Pablo Palazon',
    'author_email': 'ppalazon@antara.ws',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ppalazon/ecs-tasks-ops',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)
